/* Test of setting a dynamic button box-style.
 * There's almost certainly a better way to do this...! */

#include <stdio.h>
#include <unistd.h>

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>

static Fl_Double_Window *main_win = NULL;
static Fl_Color bg_light = FL_BACKGROUND_COLOR;
static Fl_Color bg_dark = FL_BLACK;
static Fl_Color bg_over = FL_BACKGROUND_COLOR;

class style_button : public Fl_Button
{
    public:
    style_button (int x, int y, int w, int h, const char *L = 0) :
       Fl_Button(x, y, w, h, L),
       flat_button (false),
       is_dark(false),
       cur_box(FL_THIN_UP_BOX)
       {clear_visible_focus();}

    // Event handling
    int handle (int ev)
    {
    int res = Fl_Button::handle(ev);
    switch (ev)
    {
        case FL_ENTER:
        if (flat_button != false)
        {
            color(bg_over);
            redraw();
        }
        res = 1;
        break;

        case FL_LEAVE:
        if (flat_button != false)
        {
            if (is_dark) color(bg_dark);
            else color(bg_light);
            redraw();
        }
        res = 1;
        break;

        default:
        break;
    }
    return res;
    } /* end of handle() method */

    static void set_type (const Fl_Boxtype bt)
    {
        if (bt == FL_FLAT_BOX)
        {
            g_box = FL_FLAT_BOX;
        }
        else
        {
            g_box = FL_THIN_UP_BOX;
        }
    }

    static void set_dark (const int bt)
    {
        if (bt)
        {
            g_dark = true;
        }
        else
        {
            g_dark = false;
        }
    }

    protected:
    void draw ()
    {
        if (is_dark != g_dark)
        {
            is_dark = g_dark;
            if (is_dark)
            {
                color (bg_dark);
                labelcolor (bg_light);
            }
            else
            {
                color(bg_light);
                labelcolor (FL_BLACK);
            }
        }
        if (g_box != cur_box)
        {
            cur_box = g_box;
            box (cur_box);
            if (cur_box == FL_FLAT_BOX)
            {
                flat_button = true;
            }
            else
            {
                flat_button = false;
                if (is_dark) color(bg_dark);
                else color(bg_light);
            }
        }
        Fl_Button::draw();
    }

    private:
    bool flat_button;
    bool is_dark;
    Fl_Boxtype cur_box;
    static Fl_Boxtype g_box;
    static bool g_dark;

}; // style_button

Fl_Boxtype style_button::g_box = FL_THIN_UP_BOX;
bool style_button::g_dark = false;

static void style_cb (Fl_Widget *pw, void *)
{
    static int flat = 0;
    int res = flat ^ 1;
    if (res)
    {
        style_button::set_type (FL_FLAT_BOX);
        pw->label ("Raised");
    }
    else
    {
        style_button::set_type (FL_THIN_UP_BOX);
        pw->label ("Flat");
    }
    flat = res;
    main_win->redraw();
} // style_cb

static void dark_cb (Fl_Widget *pw, void *)
{
    static int flat = 0;
    int res = flat ^ 1;
    if (res)
    {
        pw->label ("Light");
        bg_over = fl_lighter (bg_dark);
        main_win->color (bg_dark);
    }
    else
    {
        pw->label ("Dark");
        bg_over = fl_darker (bg_light);
        main_win->color (bg_light);
    }
    flat = res;
    style_button::set_dark (res);
    main_win->redraw();
} // dark_cb

static void quit_cb (Fl_Widget *, void *)
{
    if (main_win) main_win->hide();
} // quit_cb

int main(int argc, char **argv)
{
    const int wd = 512;
    const int ht = 512;

    bg_dark = fl_rgb_color (40); // set a darkish grey background
    bg_over = fl_darker (bg_light);

    main_win = new Fl_Double_Window (wd, ht, "Test Boxes");
    main_win->begin();
    main_win->box (FL_FLAT_BOX);

    style_button toggle_flat (10, 10, 70, 25, "Flat");
    toggle_flat.callback (style_cb, NULL);

    style_button toggle_dark (10, 45, 70, 25, "Dark");
    toggle_dark.callback (dark_cb, NULL);

    // Make some other buttons...
    int xx = 90;
    int yy = 10;
    for (int idx = 0; idx < 56; ++idx)
    {
        style_button *dummy = new style_button (xx, yy, 70, 25);
        char label[16];
        sprintf (label, "Button %d", (idx + 1));
        dummy->copy_label (label);
        yy += 35;
        if (yy >= (ht - 25))
        {
            yy = 10;
            xx += 80;
        }
    }

    style_button qt_bt ((wd - 70 - 10), (ht - 25 - 10), 70, 25, "Quit");
    qt_bt.callback (quit_cb, NULL);

    main_win->end();
    main_win->show (argc, argv);

    return Fl::run();
} // main

/* End of File */
